@@ -34,7 +34,7 @@ class CloneModelAdminMixin(object):
                 clone.save()
                 self.message_user(
                     request,
-                    _("Duplication successful, redirected to cloned: {}".format(clone)),
+                    _("Duplication successful, redirected to cloned: {}").format(clone),
                 )
                 cloned_admin_url = reverse(
                     "admin:{0}_{1}_change".format(
@@ -57,7 +57,7 @@ class CloneModelAdminMixin(object):
         if clone_obj_ids:
             self.message_user(
                 request,
-                _("Successfully created: {} new duplicates".format(len(clone_obj_ids))),
+                _("Successfully created: {} new duplicates").format(len(clone_obj_ids)),
             )
 
     make_clone.short_description = _("Duplicate selected %(verbose_name_plural)s")
