@@ -1,20 +1,21 @@
+import itertools
 from itertools import repeat
-from typing import List, Optional, Dict
+from typing import Dict, List, Optional
 
 from conditional import conditional
 from django.core.checks import Error
 from django.core.exceptions import ValidationError
-from django.db import transaction, models, IntegrityError, connections
+from django.db import IntegrityError, connections, models, transaction
 from django.db.models import SlugField
 from django.utils.text import slugify
 
 from model_clone.apps import ModelCloneConfig
 from model_clone.utils import (
     clean_value,
-    transaction_autocommit,
-    get_unique_value,
     context_mutable_attribute,
     get_fields_and_unique_fields_from_cls,
+    get_unique_value,
+    transaction_autocommit,
 )
 
 
@@ -154,7 +155,7 @@ class CloneMixin(object):
                         "Please provide either "
                         + '"_clone_m2o_or_o2m_fields"'
                         + " or "
-                        + '"_clone_excluded_m2o_or_o2m_fields" for {}'.format(
+                        + '"_clone_excluded_m2o_or_o2m_fields" for model {}'.format(
                             cls.__name__
                         )
                     ),
@@ -169,7 +170,9 @@ class CloneMixin(object):
                     "Conflicting configuration.",
                     hint=(
                         'Please provide either "_clone_o2o_fields"'
-                        + ' or "_clone_excluded_o2o_fields" for {}'.format(cls.__name__)
+                        + ' or "_clone_excluded_o2o_fields" for model {}'.format(
+                            cls.__name__
+                        )
                     ),
                     obj=cls,
                     id="{}.E002".format(ModelCloneConfig.name),
@@ -185,7 +188,7 @@ class CloneMixin(object):
         pass
 
     @transaction.atomic
-    def make_clone(self, attrs=None, sub_clone=False):
+    def make_clone(self, attrs=None, sub_clone=False, using=None):
         """Creates a clone of the django model instance.
 
         :param attrs: Dictionary of attributes to be replaced on the cloned object.
@@ -193,8 +196,11 @@ class CloneMixin(object):
         :param sub_clone: Internal boolean used to detect cloning sub objects.
         :type sub_clone: bool
         :rtype: :obj:`django.db.models.Model`
+        :param using: The database alias used to save the created instances.
+        :type using: str
         :return: The model instance that has been cloned.
         """
+        using = using or self._state.db or self.__class__._default_manager.db
         attrs = attrs or {}
         if not self.pk:
             raise ValidationError(
@@ -203,31 +209,35 @@ class CloneMixin(object):
                 )
             )
         if sub_clone:
-            duplicate = self
-            duplicate.pk = None
+            duplicate = self  # pragma: no cover
+            duplicate.pk = None  # pragma: no cover
         else:
-            duplicate = self._create_copy_of_instance(self)
+            duplicate = self._create_copy_of_instance(self, using=using)
 
         for name, value in attrs.items():
             setattr(duplicate, name, value)
 
-        duplicate.save()
+        duplicate.save(using=using)
+
+        duplicate = self.__duplicate_m2o_fields(duplicate, using=using)
+        duplicate = self.__duplicate_o2o_fields(duplicate, using=using)
+        duplicate = self.__duplicate_o2m_fields(duplicate, using=using)
+        duplicate = self.__duplicate_m2m_fields(duplicate, using=using)
 
-        duplicate = self.__duplicate_o2o_fields(duplicate)
-        duplicate = self.__duplicate_o2m_fields(duplicate)
-        duplicate = self.__duplicate_m2o_fields(duplicate)
-        duplicate = self.__duplicate_m2m_fields(duplicate)
         return duplicate
 
-    def bulk_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
-        ops = connections[self.__class__._default_manager.db].ops
+    def bulk_clone(
+        self, count, attrs=None, batch_size=None, using=None, auto_commit=False
+    ):
+        using = using or self._state.db or self.__class__._default_manager.db
+        ops = connections[using].ops
         objs = range(count)
         clones = []
         batch_size = batch_size or max(ops.bulk_batch_size([], list(objs)), 1)
 
         with conditional(
             auto_commit,
-            transaction_autocommit(using=self.__class__._default_manager.db),
+            transaction_autocommit(using=using),
         ):
             # If count exceeds the MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS
             with conditional(
@@ -250,21 +260,23 @@ class CloneMixin(object):
 
     def parallel_clone(self, count, attrs=None, batch_size=None, auto_commit=False):
         # if this takes n time for t records
-        # t^n i.e 100 * 10ms = 1000ms to clone 100 objects.
+        # t^n i.e 100 ** 2ms = 10000ms (10s) to clone 100 objects.
         # I'll like to reduce this down to max time to clone count/batch_size i.e
-        # If it take 100ms to clone 100 objects with a db of batch_size 100
-        # If it takes 10ms to clone 10 objects i'll like to keep this down to 10ms for
-        # max_num_of_threads i.e 10 threads for 100 objects.
+        # If it take 10000ms to clone 100 objects with a db of batch_size 100
+        # If it takes 20ms to clone 10 objects I'll like to keep this down to 20ms
+        # i.e max_num_of_threads 10 for 100 objects.
         # This should run in parallel
         # Testing jit and cpython if they offer better API's.
         pass
 
     @staticmethod
-    def _create_copy_of_instance(instance, force=False, sub_clone=False):
+    def _create_copy_of_instance(instance, using=None, force=False, sub_clone=False):
         """Create a copy of an instance
 
         :param instance: The instance to be duplicated.
         :type instance: `django.db.models.Model`
+        :param using: The database alias used to save the created instances.
+        :type using: str
         :param force: Flag to skip using the current model clone declared attributes.
         :type force: bool
         :param sub_clone: Flag to skip cloning one to one fields for sub clones.
@@ -325,7 +337,10 @@ class CloneMixin(object):
                 ]
             ):
                 # Do not try to get unique value for enum type field
-                if isinstance(f, models.CharField) and not f.choices:
+                if (
+                    isinstance(f, (models.CharField, models.TextField))
+                    and not f.choices
+                ):
                     value = clean_value(value, unique_duplicate_suffix)
                     if use_unique_duplicate_suffix:
                         value = get_unique_value(
@@ -347,19 +362,19 @@ class CloneMixin(object):
                             force=True,
                             sub_clone=True,
                         )
-                        sub_instance.save()
+                        sub_instance.save(using=using)
                         value = sub_instance.pk
 
             setattr(new_instance, f.attname, value)
 
         return new_instance
 
-    def __duplicate_o2o_fields(self, duplicate):
-        """
-        Duplicate the one to one fields.
-
+    def __duplicate_o2o_fields(self, duplicate, using=None):
+        """Duplicate one to one fields.
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
+        :param using: The database alias used to save the created instances.
+        :type using: str
         :return: The duplicate instance with all the one to one fields duplicated.
         """
         for f in self._meta.related_objects:
@@ -381,20 +396,23 @@ class CloneMixin(object):
                             sub_clone=True,
                         )
                         setattr(new_rel_object, f.remote_field.name, duplicate)
-                        new_rel_object.save()
+                        new_rel_object.save(using=using)
 
         return duplicate
 
-    def __duplicate_o2m_fields(self, duplicate):
+    def __duplicate_o2m_fields(self, duplicate, using=None):
         """Duplicate one to many fields.
 
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
-        :return: The duplicate instance with all the one to many fields duplicated.
+        :param using: The database alias used to save the created instances.
+        :type using: str
+        :return: The duplicate instance with all the transcient one to many duplicated instances.
         """
-        fields = set()
 
-        for f in self._meta.related_objects:
+        for f in itertools.chain(
+            self._meta.related_objects, self._meta.concrete_fields
+        ):
             if f.one_to_many:
                 if any(
                     [
@@ -404,29 +422,42 @@ class CloneMixin(object):
                         not in self._clone_excluded_m2o_or_o2m_fields,
                     ]
                 ):
-                    fields.add(f)
+                    for item in getattr(self, f.get_accessor_name()).all():
+                        if hasattr(item, "make_clone"):
+                            try:
+                                item.make_clone(
+                                    attrs={f.remote_field.name: duplicate},
+                                    using=using,
+                                )
+                            except IntegrityError:
+                                item.make_clone(
+                                    attrs={f.remote_field.name: duplicate},
+                                    save_new=False,
+                                    sub_clone=True,
+                                    using=using,
+                                )
+                        else:
+                            new_item = CloneMixin._create_copy_of_instance(
+                                item,
+                                force=True,
+                                sub_clone=True,
+                                using=using,
+                            )
+                            setattr(new_item, f.remote_field.name, duplicate)
 
-        # Clone one to many fields
-        for field in fields:
-            for item in getattr(self, field.get_accessor_name()).all():
-                try:
-                    item.make_clone(attrs={field.remote_field.name: duplicate})
-                except IntegrityError:
-                    item.make_clone(
-                        attrs={field.remote_field.name: duplicate}, sub_clone=True
-                    )
+                            new_item.save(using=using)
 
         return duplicate
 
-    def __duplicate_m2o_fields(self, duplicate):
+    def __duplicate_m2o_fields(self, duplicate, using=None):
         """Duplicate many to one fields.
 
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
+        :param using: The database alias used to save the created instances.
+        :type using: str
         :return: The duplicate instance with all the many to one fields duplicated.
         """
-        fields = set()
-
         for f in self._meta.concrete_fields:
             if f.many_to_one:
                 if any(
@@ -436,25 +467,27 @@ class CloneMixin(object):
                         and f.name not in self._clone_excluded_m2o_or_o2m_fields,
                     ]
                 ):
-                    fields.add(f)
+                    item = getattr(self, f.name)
+                    if hasattr(item, "make_clone"):
+                        try:
+                            item_clone = item.make_clone(using=using)
+                        except IntegrityError:
+                            item_clone = item.make_clone(sub_clone=True)
+                    else:
+                        item.pk = None  # pragma: no cover
+                        item_clone = item.save(using=using)  # pragma: no cover
 
-        # Clone many to one fields
-        for field in fields:
-            item = getattr(self, field.name)
-            try:
-                item_clone = item.make_clone()
-            except IntegrityError:
-                item_clone = item.make_clone(sub_clone=True)
-
-            setattr(duplicate, field.name, item_clone)
+                    setattr(duplicate, f.name, item_clone)
 
         return duplicate
 
-    def __duplicate_m2m_fields(self, duplicate):
+    def __duplicate_m2m_fields(self, duplicate, using=None):
         """Duplicate many to many fields.
 
         :param duplicate: The transient instance that should be duplicated.
         :type duplicate: `django.db.models.Model`
+        :param using: The database alias used to save the created instances.
+        :type using: str
         :return: The duplicate instance with all the many to many fields duplicated.
         """
         fields = set()
@@ -468,6 +501,7 @@ class CloneMixin(object):
                 ]
             ):
                 fields.add(f)
+
         for f in self._meta.related_objects:
             if f.many_to_many:
                 if any(
@@ -503,15 +537,20 @@ class CloneMixin(object):
                 for item in objs:
                     if hasattr(through, "make_clone"):
                         try:
-                            item.make_clone(attrs={field_name: duplicate})
+                            item.make_clone(
+                                attrs={field_name: duplicate},
+                                using=using,
+                            )
                         except IntegrityError:
                             item.make_clone(
-                                attrs={field_name: duplicate}, sub_clone=True
+                                attrs={field_name: duplicate},
+                                sub_clone=True,
+                                using=using,
                             )
                     else:
                         item.pk = None
                         setattr(item, field_name, duplicate)
-                        item.save()
+                        item.save(using=using)
             else:
                 destination.set(source.all())
 
