@@ -2,12 +2,13 @@ import contextlib
 import re
 
 import six
-from django.core.exceptions import ValidationError
 from django.db import models, transaction
 from django.db.transaction import TransactionManagementError
 
 
-def create_copy_of_instance(instance, exclude=(), save_new=True, attrs=None):
+def create_copy_of_instance(
+    instance, attrs=None, exclude=(), save_new=True, using=None
+):
     """
     Clone an instance of `django.db.models.Model`.
 
@@ -17,6 +18,8 @@ def create_copy_of_instance(instance, exclude=(), save_new=True, attrs=None):
     :type exclude: list|set
     :param save_new: Save the model instance after duplication calling .save().
     :type save_new: bool
+    :param using: The database alias used to save the created instances.
+    :type using: str
     :param attrs: Kwargs of field and value to set on the duplicated instance.
     :type attrs: dict
     :return: The new duplicated instance.
@@ -42,13 +45,17 @@ def create_copy_of_instance(instance, exclude=(), save_new=True, attrs=None):
 
     defaults = {}
     attrs = attrs or {}
+    default_db_alias = instance._state.db or instance.__class__._default_manager.db
+    using = using or default_db_alias
     fields = instance.__class__._meta.concrete_fields
 
     if not isinstance(attrs, dict):
         try:
             attrs = dict(attrs)
         except (TypeError, ValueError):
-            raise ValueError("Invalid: Expected attrs to be a dict or iterable.")
+            raise ValueError(
+                "Invalid: Expected attrs to be a dict or iterable of key and value tuples."
+            )
 
     for f in fields:
         if all(
@@ -81,14 +88,13 @@ def create_copy_of_instance(instance, exclude=(), save_new=True, attrs=None):
         )
     ]
 
-    try:
-        # Run the unique validation before creating the instance.
+    # Bug with django using full_clean on a different db
+    if using == default_db_alias:
+        # Validate the new instance on the same database
         new_obj.full_clean(exclude=exclude)
-    except ValidationError as e:
-        raise ValidationError(", ".join(e.messages))
 
     if save_new:
-        new_obj.save()
+        new_obj.save(using=using)
 
     return new_obj
 
@@ -140,18 +146,16 @@ def context_mutable_attribute(obj, key, value):
     """
     Context manager that modifies an obj temporarily.
     """
-    default = None
-    is_set = hasattr(obj, key)
-    if is_set:
-        default = getattr(obj, key)
+    attribute_exists = hasattr(obj, key)
+    default = getattr(obj, key, None)
     try:
         setattr(obj, key, value)
         yield
     finally:
-        if not is_set and hasattr(obj, key):
-            del obj[key]
-        else:
+        if attribute_exists:
             setattr(obj, key, default)
+        else:
+            delattr(obj, key)  # pragma: no cover
 
 
 def get_value(value, suffix, transform, max_length, index):
@@ -162,7 +166,7 @@ def get_value(value, suffix, transform, max_length, index):
     duplicate_suffix = " {} {}".format(suffix, index)
     total_length = len(value + duplicate_suffix)
 
-    if total_length > max_length:
+    if max_length is not None and total_length > max_length:
         # Truncate the value to max_length - suffix length.
         value = value[: max_length - len(duplicate_suffix)]
 
@@ -235,7 +239,7 @@ def get_fields_and_unique_fields_from_cls(
                 and not force
                 and getattr(f, "one_to_one", False)
             ):
-                valid = f.name not in clone_excluded_o2o_fields
+                valid = f.name not in clone_excluded_o2o_fields  # pragma: no cover
             else:
                 valid = True
 
